@@ -39,10 +39,10 @@ class FormRenderer(DefaultFormRenderer):
         return context
 
     def _amend_collection(self, context):
+        context = super()._amend_collection(context)
         context.update({
             'add_collection_button': 'formset/uikit/buttons/add_collection.html',
             'remove_collection_button': 'formset/uikit/buttons/remove_collection.html',
-            'css_classes': self.collection_css_classes,
         })
         return context
 
